@@ -1,29 +1,47 @@
 # $Id: Currency.pm 1830 2007-05-05 01:34:36Z claco $
 package Data::Currency;
+{
+  $Data::Currency::VERSION = '0.05';
+}
 use strict;
 use warnings;
 use overload
-    '0+'     => sub {shift->value},
-    'bool'   => sub {shift->value},
-    '""'     => sub {shift->stringify},
-    fallback => 1;
-use vars qw/$VERSION/;
+  '0+'     => sub { shift->value },
+  'bool'   => sub { shift->value },
+  '""'     => sub { shift->stringify },
+  '+'      => \&_add,
+  '-'      => \&_substract,
+  '*'      => \&_multiply,
+  '/'      => \&_divide,
+  '%'      => \&_modulo,
+  '<=>'    => \&_three_way_compare,
+  'cmp'    => \&_three_way_compare,
+  'abs'    => \&_abs,
+  'int'    => \&_int,
+  'neg'    => \&_negate,
+  fallback => 1;
 
-$VERSION = '0.04002';
+# TODO Operations
+#    '+='    => \&add_in_place,
+#    '-='    => \&subtract_in_place,
+#    '*='    => \&multiply_in_place,
+#    '/='    => \&divide_in_place,
 
 BEGIN {
     use base qw/Class::Accessor::Grouped/;
     use Locale::Currency ();
     use Locale::Currency::Format;
-    use Scalar::Util ();
+    use Scalar::Util     ();
     use Class::Inspector ();
     use Carp;
 
-    __PACKAGE__->mk_group_accessors('inherited', qw/
-        format value converter converter_class
-    /);
-    __PACKAGE__->mk_group_accessors('component_class', qw/converter_class/);
-};
+    __PACKAGE__->mk_group_accessors(
+        'inherited', qw/
+          format value converter converter_class
+          /
+    );
+    __PACKAGE__->mk_group_accessors( 'component_class', qw/converter_class/ );
+}
 
 __PACKAGE__->converter_class('Finance::Currency::Convert::WebserviceX');
 __PACKAGE__->value(0);
@@ -33,159 +51,280 @@ __PACKAGE__->format('FMT_COMMON');
 my %codes;
 
 sub new {
-    my ($class, $value, $code, $format) = @_;
+    my ( $class, $value, $code, $format ) = @_;
     my $self = bless {}, $class;
 
-    if (ref $value eq 'HASH') {
-        foreach my $key (keys %{$value}) {
-            $self->$key($value->{$key}) if defined $value->{$key};
-        };
-    } else {
-        if (defined $value){
-            $self->value($value)
-        };
+    if ( ref $value eq 'HASH' ) {
+        foreach my $key ( keys %{$value} ) {
+            $self->$key( $value->{$key} ) if defined $value->{$key};
+        }
+    }
+    else {
+        if ( defined $value ) {
+            $self->value($value);
+        }
         if ($code) {
-            $self->code($code)
-        };
+            $self->code($code);
+        }
         if ($format) {
-            $self->format($format)
-        };
-    };
+            $self->format($format);
+        }
+    }
 
     return $self;
-};
+}
 
 sub code {
     my $self = shift;
 
-    if (scalar @_) {
+    if ( scalar @_ ) {
         my $code = shift;
 
         croak "Invalid currency code: $code"
-            unless _is_currency_code($code);
+          unless _is_currency_code($code);
 
-        $self->set_inherited('code', $code);
-    };
+        $self->set_inherited( 'code', $code );
+    }
 
     return $self->get_inherited('code');
-};
+}
 
 sub convert {
-    my ($self, $to) = @_;
+    my ( $self, $to ) = @_;
     my $class = Scalar::Util::blessed($self);
-    my $from = $self->code;
+    my $from  = $self->code;
 
     $to ||= '';
-    if (uc($from) eq uc($to)) {
+    if ( uc($from) eq uc($to) ) {
         return $self;
-    };
+    }
 
-    croak 'Invalid currency code source: ' . ($from || 'undef')
-        unless _is_currency_code($from);
+    croak 'Invalid currency code source: ' . ( $from || 'undef' )
+      unless _is_currency_code($from);
 
-    croak 'Invalid currency code target: ' . ($to || 'undef')
-        unless _is_currency_code($to);
+    croak 'Invalid currency code target: ' . ( $to || 'undef' )
+      unless _is_currency_code($to);
 
-    if (!$self->converter) {
-        $self->converter($self->converter_class->new)
-    };
+    if ( !$self->converter ) {
+        $self->converter( $self->converter_class->new );
+    }
 
-    return $class->new(
-        $self->converter->convert($self->value, $from, $to) || 0,
-        $to,
-        $self->format
-    );
-};
+    return $class->new( $self->converter->convert( $self->value, $from, $to )
+          || 0,
+        $to, $self->format );
+}
 
 sub name {
     my $self = shift;
-    my $name = Locale::Currency::code2currency($self->code);
+    my $name = Locale::Currency::code2currency( $self->code );
 
     ## Fix for older Locale::Currency w/mispelled Candian
     $name =~ s/Candian/Canadian/;
 
     return $name;
-};
+}
 
 *as_string = \&stringify;
 
 sub stringify {
-    my $self = shift;
+    my $self   = shift;
     my $format = shift || $self->format;
-    my $code = $self->code;
-    my $value = $self->value;
+    my $code   = $self->code;
+    my $value  = $self->value;
 
-    if (!$format) {
+    if ( !$format ) {
         $format = 'FMT_COMMON';
-    };
+    }
 
     ## funky eval to get string versions of constants back into the values
-    eval '$format = Locale::Currency::Format::' .  $format;
+    eval '$format = Locale::Currency::Format::' . $format;
 
-    croak 'Invalid currency code:  ' . ($code || 'undef')
-        unless _is_currency_code($code);
+    croak 'Invalid currency code:  ' . ( $code || 'undef' )
+      unless _is_currency_code($code);
 
     return _to_utf8(
-        Locale::Currency::Format::currency_format($code, $value, $format)
-    );
-};
+        Locale::Currency::Format::currency_format( $code, $value, $format ) );
+}
+
+sub as_float {
+    my $self  = shift;
+    my $radix = $self->_radix;
+    return sprintf( "%.0${radix}f", $self->value );
+}
 
 sub _is_currency_code {
     my $value = defined $_[0] ? uc(shift) : '';
 
-    return unless ($value =~ /^[A-Z]{3}$/);
+    return unless ( $value =~ /^[A-Z]{3}$/ );
 
-    if (! keys %codes) {
-        %codes = map {uc($_) => uc($_)} Locale::Currency::all_currency_codes();
-    };
+    if ( !keys %codes ) {
+        %codes =
+          map { uc($_) => uc($_) } Locale::Currency::all_currency_codes();
+    }
     return exists $codes{$value};
-};
+}
 
 sub _to_utf8 {
     my $value = shift;
 
-    if ($] >= 5.008) {
+    if ( $] >= 5.008 ) {
         require utf8;
         utf8::upgrade($value);
-    };
+    }
 
     return $value;
-};
+}
 
 sub get_component_class {
-    my ($self, $field) = @_;
+    my ( $self, $field ) = @_;
 
     return $self->get_inherited($field);
-};
+}
 
 sub set_component_class {
-    my ($self, $field, $value) = @_;
+    my ( $self, $field, $value ) = @_;
 
     if ($value) {
-        if (!Class::Inspector->loaded($value)) {
+        if ( !Class::Inspector->loaded($value) ) {
             eval "use $value";
 
             croak "The $field $value could not be loaded: $@" if $@;
-        };
-    };
+        }
+    }
 
-    $self->set_inherited($field, $value);
+    $self->set_inherited( $field, $value );
 
     return;
-};
+}
+
+sub _radix {
+    my $self = shift;
+    return Locale::Currency::Format::decimal_precision( $self->code ) || 0;
+}
+
+sub _add {
+    my ( $self, $other ) = @_;
+
+    if ( Scalar::Util::blessed($other) && $other->isa(__PACKAGE__) ) {
+        croak "Unable to perform math operation with different currency types"
+          if $self->code ne $other->code;
+
+        $other = $other->value;
+    }
+
+    __PACKAGE__->new( $self->value + $other, $self->code, $self->format );
+}
+
+sub _substract {
+    my ( $self, $other, $reversed ) = @_;
+
+    if ( Scalar::Util::blessed($other) && $other->isa(__PACKAGE__) ) {
+        croak "Unable to perform math operation with different currency types"
+          if $self->code ne $other->code;
+
+        $other = $other->value;
+    }
+
+    my $new_value = $reversed ? $other - $self->value : $self->value - $other;
+    __PACKAGE__->new( $new_value, $self->code, $self->format );
+}
+
+sub _multiply {
+    my ( $self, $other ) = @_;
+
+    if ( Scalar::Util::blessed($other) && $other->isa(__PACKAGE__) ) {
+        croak "Unable to perform math operation with different currency types"
+          if $self->code ne $other->code;
+
+        $other = $other->value;
+    }
+
+    __PACKAGE__->new( $self->value * $other, $self->code, $self->format );
+}
+
+sub _divide {
+    my ( $self, $other, $reversed ) = @_;
+
+    if ( Scalar::Util::blessed($other) && $other->isa(__PACKAGE__) ) {
+        croak "Unable to perform math operation with different currency types"
+          if $self->code ne $other->code;
+
+        $other = $other->value;
+    }
+
+    croak "Illegal division by zero"
+      if $other == 0
+      or ( $reversed and $self->value == 0 );
+
+    my $new_value = $reversed ? $other / $self->value : $self->value / $other;
+    __PACKAGE__->new( $new_value, $self->code, $self->format );
+}
+
+sub _modulo {
+    my ( $self, $other, $reversed ) = @_;
+
+    if ( Scalar::Util::blessed($other) && $other->isa(__PACKAGE__) ) {
+        croak "Unable to perform math operation with different currency types"
+          if $self->code ne $other->code;
+
+        $other = $other->value;
+    }
+
+    croak "Illegal modulus zero"
+      if $other == 0
+      or ( $reversed and $self->value == 0 );
+
+    my $new_value = $reversed ? $other % $self->value : $self->value % $other;
+    __PACKAGE__->new( $new_value, $self->code, $self->format );
+}
+
+sub _three_way_compare {
+    my ( $self, $other, $reversed ) = @_;
+
+    if ( Scalar::Util::blessed($other) && $other->isa(__PACKAGE__) ) {
+        croak "Unable to perform comparison with different currency types"
+          if $self->code ne $other->code;
+
+        $other = $other->value;
+    }
+
+    return $reversed ? $other <=> $self->value : $self->value <=> $other;
+}
+
+sub _abs {
+    my $self = shift;
+    __PACKAGE__->new( abs( $self->value ), $self->code, $self->format );
+}
+
+sub _int {
+    my $self = shift;
+    __PACKAGE__->new( int( $self->value ), $self->code, $self->format );
+}
+
+sub _negate {
+    my $self = shift;
+    __PACKAGE__->new( -$self->value, $self->code, $self->format );
+}
 
 1;
+
 __END__
 
+=pod
+
 =head1 NAME
 
-Data::Currency - Container class for currency conversion/formatting
+Data::Currency
+
+=head1 VERSION
+
+version 0.05
 
 =head1 SYNOPSIS
 
     use Data::Currency;
 
-    my $price = Data::Currency->new(1.2. 'USD');
+    my $price = Data::Currency->new(1.2, 'USD');
     print $price;            # 1.20 USD
     print $price->code;      # USD
     print $price->format;    # FMT_SYMBOL
@@ -207,6 +346,14 @@ The Data::Currency module provides basic currency formatting and conversion:
 Each Data::Currency object will stringify to the original value except in string
 context, where it stringifies to the format specified in C<format>.
 
+=head1 NAME
+
+Data::Currency - Container class for currency conversion/formatting
+
+=head1 VERSION
+
+version 0.05
+
 =head1 CONSTRUCTOR
 
 =head2 new
@@ -332,6 +479,11 @@ Sames as C<as_string>.
 
 Returns the current objects value as a formatted currency string.
 
+=head2 as_string
+
+Returns the value formatted as float using decimal places specified by currency
+code
+
 =head2 value
 
 Returns the original price value given to C<new>.
@@ -372,7 +524,13 @@ L<Finance::Currency::Convert::WebserviceX>
 
 =head1 AUTHOR
 
-    Christopher H. Laco
-    CPAN ID: CLACO
-    claco@chrislaco.com
-    http://today.icantfocus.com/blog/
+Christopher H. Laco <claco _at_ chrislaco.com>, Mariano Wahlmann <dichoso _at_ gmail.com>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2013 by Christopher H. Laco, Mariano Wahlmann.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
